@@ -51,6 +51,8 @@ instance Pretty Attribute where
   pretty (AtAlpha index) = pretty "α" <> pretty index
   pretty AtRho = pretty "ρ"
   pretty AtPhi = pretty "φ"
+  pretty AtDelta = pretty "Δ"
+  pretty AtLambda = pretty "λ"
   pretty (AtMeta meta) = prettyMeta meta
 
 instance Pretty (Formatted Binding) where
@@ -69,6 +71,7 @@ instance Pretty (Formatted Binding) where
         BiVoid attr -> attr : voids bds
         _ -> []
       inlineVoids :: [Attribute] -> Doc ann
+      inlineVoids [] = pretty attr <+> prettyArrow
       inlineVoids voids' = pretty attr <> lparen <> hsep (punctuate comma (map pretty voids')) <> rparen <+> prettyArrow
   pretty (Formatted (mode, BiTau attr expr)) = pretty attr <+> prettyArrow <+> pretty (Formatted (mode, expr))
   pretty (Formatted (_, BiMeta meta)) = prettyMeta meta
